using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
	/// <summary>
	/// Implement a full CardGroup name (CardName.CardGroupName)
	/// </summary>
    //
    //********************************************************************************
    [TypeConverter(typeof(CrdGrpFullName.CrdGrpFullNameConverter))]
    public class CrdGrpFullName {
        /// <summary>Name of the card</summary>
        private string      m_strCrdName;
        /// <summary>Name of the card group</summary>
        private string      m_strCrdGrpName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="strCrdName">       Card name</param>
        /// <param name="strCrdGrpName">    Card Group name</param>
        //
        //********************************************************************************
        public CrdGrpFullName(string strCrdName, string strCrdGrpName) {
            m_strCrdName    = (strCrdName == null) ? "" : strCrdName.ToUpper();
            m_strCrdGrpName = (strCrdGrpName == null) ? "" : strCrdGrpName.ToUpper();
        }

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //********************************************************************************
        public CrdGrpFullName() : this("","") {
        }

        //********************************************************************************
        //
        /// <summary>
        /// CardName:                  Name of the card
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(CrdGrpControlUtil.CardNameEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the card")]
        public string CardName {
            get {
                return(m_strCrdName);
            }
            set {
                m_strCrdName = (value == null) ? "" : value.ToUpper();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CardGroupName:                  Name of the card group
        /// </summary>
        //
        //********************************************************************************
        [Editor(typeof(CrdGrpControlUtil.CardGroupNameEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Description("Name of the card group")]
        public string CardGroupName {
            get {
                return(m_strCrdGrpName);
            }
            set {
                m_strCrdGrpName = (value == null) ? "" : value.ToUpper();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Implementation of a converter for the CrdGrpFullName type
        /// </summary>
        //
        //********************************************************************************
        public class CrdGrpFullNameConverter : TypeConverter {

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertFrom:             Determine if we can convert from a specified type
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="typeSource">   Source type</param>
            /// <returns>
            /// true if yes, false if not
            /// </returns>
            //
            //********************************************************************************
            public override bool CanConvertFrom(ITypeDescriptorContext ctx, Type typeSource) {
                bool    bRetVal;

                bRetVal = (typeSource == typeof(string)) ? true : base.CanConvertFrom(ctx, typeSource);
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CanConvertTo:                   Determine if we can convert to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// true if yes, false if not
            ///</returns>
            //
            //********************************************************************************
            public override bool CanConvertTo(ITypeDescriptorContext ctx, Type typeDestination) {
                bool    bRetVal;

                if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                    bRetVal = true;
                } else {
                    bRetVal = base.CanConvertTo(ctx, typeDestination);
                }
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertFrom:                    Convert a type to a CrdGrpFullName
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Object to convert</param>
            /// <returns>
            /// New CrdGrpFullName object or null
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertFrom(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val) {
                object                      oRetVal;
                string                      strValue;
                string[]                    arr;

                if (val == null) {
                    oRetVal = new CrdGrpFullName();
                } else {
                    if (val.GetType() == typeof(String)) {
                        strValue = val as string;
                        if (strValue != null && strValue.Length != 0) {
                            arr         = strValue.Split('.');
                            if (arr.Length != 2) {
                                throw new ArgumentException("Invalid object", "value");
                            }
                            oRetVal = new CrdGrpFullName(arr[0], arr[1]);
                        } else {
                            oRetVal = new CrdGrpFullName();
                        }
                    } else {
                        oRetVal = base.ConvertFrom(ctx, cultureInfo, val);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ConvertTo:                      Convert a CrdGrpFullName object to a specified type
            /// </summary>
            /// <param name="ctx">              Context</param>
            /// <param name="cultureInfo">      Culture info</param>
            /// <param name="val">              Value to convert</param>
            /// <param name="typeDestination">  Destination type</param>
            /// <returns>
            /// New object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object ConvertTo(ITypeDescriptorContext ctx, System.Globalization.CultureInfo cultureInfo, object val, Type typeDestination) {
                object                                  oRetVal;
                Type[]                                  arrParamType;
                ArrayList                               arrParam;
                CrdGrpFullName                          cgrpFullName;
                System.Reflection.ConstructorInfo       cinfo;

                if (typeDestination == null) {
                    throw new ArgumentNullException("destinationType");
                }
                if (val == null) {
                    oRetVal = "";
                } else {
                    if (val.GetType() != typeof(CrdGrpFullName)) {
                        throw new ArgumentException("Invalid object", "value");
                    }
                    if (typeDestination == typeof(string)) {
                        cgrpFullName = (CrdGrpFullName)val;
                        oRetVal      = cgrpFullName.CardName + "." + cgrpFullName.CardGroupName;
                    } else if (typeDestination == typeof(System.ComponentModel.Design.Serialization.InstanceDescriptor)) {
                        cgrpFullName      = (CrdGrpFullName)val;
                        arrParamType     = new Type [2];
                        arrParamType[0]  = typeof(string);
                        arrParamType[1]  = typeof(string);
                        cinfo            = cgrpFullName.GetType().GetConstructor(arrParamType);
                        arrParam         = new ArrayList(2);
                        arrParam.Add(cgrpFullName.CardName);
                        arrParam.Add(cgrpFullName.CardGroupName);
                        oRetVal = new System.ComponentModel.Design.Serialization.InstanceDescriptor(cinfo, arrParam);
                    } else {
                        oRetVal = base.ConvertTo(ctx, cultureInfo, val, typeDestination);
                    }
                }
                return(oRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CreateInstance:             Creates an instance of a CrdGrpFullName object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="propValues">   Dictionary of the argument values</param>
            /// <returns>
            /// New CrdGrpFullName object or null if error
            /// </returns>
            //
            //********************************************************************************
            public override object CreateInstance(ITypeDescriptorContext ctx, System.Collections.IDictionary propValues) {
                object  objRetVal;

                objRetVal = new CrdGrpFullName(propValues["CardName"] as string,
                                               propValues["CardGroupName"] as string);
                return(objRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetCreateInstanceSupported: Returns true to indicate we support instance creation
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetCreateInstanceSupported(ITypeDescriptorContext context) {
                return(true);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetProperties:              Gets the list of properties which are implemented by the object
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <param name="val">          Object</param>
            /// <param name="attributes">   Attributes</param>
            /// <returns>
            /// List of properties
            /// </returns>
            //
            //********************************************************************************
            public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext ctx, object val, System.Attribute[] attributes) {
                PropertyDescriptorCollection    colRetVal;
                string[]                        arrPropName;

                colRetVal      = TypeDescriptor.GetProperties(typeof(CrdGrpFullName), attributes);
                arrPropName    = new string[2];
                arrPropName[0] = "CardName";
                arrPropName[1] = "CardGroupName";
                colRetVal      = colRetVal.Sort(arrPropName);
                return(colRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetPropertiesSupported:     Return true to indicate we support properties
            /// </summary>
            /// <param name="context">      Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetPropertiesSupported(ITypeDescriptorContext context) {
                return(true);
            }
            
        } // CrdGrpFullNameConverter class
	} // CrdGrpFullName class
} // Namespace
