#if VC8
//**********************************************************************
//
//    Module:             ASPNetConnect.cs
//
//    Author:             Maher Najah
//    Version:            1.00.00
//    Creation:           2004-10-18
//    Modification:       2004-10-18
//
//    Description:        Implementing Rep++ ASP .Net Web Wizard entry object
//
//**********************************************************************
using System.Runtime.InteropServices;
using EnvDTE;
using System.Windows.Forms;
using System;
using RepPP.VSNetWizard;
using System.Collections.Generic;
using Microsoft.VisualStudio.TemplateWizard;

namespace RepPP.VSNetWizard {
	internal abstract class ConnectBase : IWizard {
        /// <summary>Wizard main form</summary>
        internal frmMainBase                    m_frmMain;
        /// <summary>Wizard kind</summary>
        protected WizardRunKind                 m_eWizKind;
        /// <summary>Template file name</summary>
        protected string                        m_strTemplateFileName;
        //**********************************************************************
        //
        /// <summary>
        /// RunStarted:                             Called when the wizard is started        
        /// </summary>
        /// <param name="automationObject">         </param>
        /// <param name="replacementsDictionary">   </param>
        /// <param name="runKind">                  </param>
        /// <param name="customParams">             </param>
        //       
        //**********************************************************************        
        public void RunStarted(object                       automationObject,
                               Dictionary<string, string>   replacementsDictionary,
                               WizardRunKind                runKind,
                               object[]                     customParams) {
            string              strPrjName;
            string              strPrjDirName;
            ProgramLanguageE    eLang;
            GenerationModeE     eGenMode;
            DTE                 dte;
            Project             project;

            m_eWizKind              = runKind;
            dte                     = automationObject as DTE;
            m_strTemplateFileName   = customParams[0].ToString();
            if (m_strTemplateFileName.IndexOf("\\CSharp\\") != -1) {
                eLang = ProgramLanguageE.sdCSharp;
            } else {
                eLang = ProgramLanguageE.sdVisualBasic;
            }
            if (runKind == WizardRunKind.AsNewItem) {
                project         = ((object[])dte.ActiveSolutionProjects)[0] as Project;
                strPrjName      = project.Name;
                strPrjDirName   = project.FullName;
                eGenMode        = GenerationModeE.sdItem;
            } else {
                strPrjName      = replacementsDictionary["$safeprojectname$"];
                strPrjDirName   = replacementsDictionary["$destinationdirectory$"];
                eGenMode        = GenerationModeE.sdApplication;
            }
            m_frmMain = CreateMainForm(dte,
                                       strPrjName,
                                       strPrjDirName,
                                       eLang,
                                       eGenMode);
            m_frmMain.ShowDialog();
            if (m_frmMain.DialogResult == System.Windows.Forms.DialogResult.Cancel) {
                m_frmMain.Dispose();
                throw new WizardCancelledException("the operation is cancelled");
            }
        }
        //**********************************************************************
        //
        /// <summary>
        /// BeforeOpeningFile:          Called before opening any item that has the 
        ///                             OpenInEditor attribute.
        /// </summary>
        /// <param name="projectItem">  Project item</param>
        //       
        //**********************************************************************
        public void BeforeOpeningFile(ProjectItem projectItem) {
        }
        //**********************************************************************
        //
        /// <summary>
        /// ProjectFinishedGenerating:      Called after finishing generating the ptoject
        /// </summary>
        /// <param name="project">          Project</param>
        //       
        //**********************************************************************
        public void ProjectFinishedGenerating(Project project) {
            try {
                if (m_frmMain.DialogResult == DialogResult.OK) {
                    // This is necessary if the project name already exists and the user
                    // have choose another before the creation of the project name
                    m_frmMain.ProjectName       = project.Name;
                    m_frmMain.ProjectFullName   = project.FullName;
                    m_frmMain.Generate();
                }
            } finally {
                m_frmMain.Dispose();
            }
        }
        //**********************************************************************
        //
        /// <summary>
        /// ProjectItemFinishedGenerating:      This method is only called for item templates,
        ///                                     not for project templates.
        /// </summary>
        /// <param name="projectItem">          Project item</param>
        //       
        //**********************************************************************        
        public void ProjectItemFinishedGenerating(ProjectItem projectItem) {
        }
        //**********************************************************************
        //
        /// <summary>
        /// ProjectItemFinishedGenerating:      Called after the project is created
        /// </summary>
        /// <param name="projectItem">          Project item</param>
        //       
        //**********************************************************************        
        public void RunFinished() {
        }
        //**********************************************************************
        //
        /// <summary>
        /// ShouldAddProjectItem:   This method is only called for item templates,
        //                          not for project templates.
        /// </summary>
        /// <param name="filePath"> File full name</param>
        //       
        //**********************************************************************        
        public bool ShouldAddProjectItem(string filePath) {
            bool    bRetVal = true;

            if (m_eWizKind == WizardRunKind.AsNewItem) {
                try {
                    if (m_frmMain.DialogResult == DialogResult.OK) {
                        m_frmMain.Generate();
                    }
                } finally {
                    m_frmMain.Dispose();
                    bRetVal = false;
                }
            }
            return(bRetVal);
        } 
        //**********************************************************************
        //
        /// <summary>
        /// CreateMainForm:                     Create main form. Method factory
        /// </summary>
        /// <param name="dte">                  Visual studio object</param>
        /// <param name="strProjectName">       Project name</param>
        /// <param name="strProjectDirName">    Project directory name</param>
        /// <param name="eLang">                Program language</param>
        /// <param name="eGenMode">             Wizard mode(item or application)</param>
        /// <returns>
        /// Wizard main form
        /// </returns>
        //       
        //**********************************************************************        
        internal abstract frmMainBase CreateMainForm(EnvDTE.DTE        dte,
                                                     string            strProjectName,
                                                     string            strProjectDirName,
                                                     ProgramLanguageE  eLang,
                                                     GenerationModeE   eGenMode);
	}
}
#endif
